#!/bin/bash
#
# Welcom to use Bigemap Server
# Copyright(c) 2020 BIGEMAP
#

set -e
help_setup() {
  echo -e "
Once the download command has completed, you will run the setup command.

Usage:

  ./bm-server-installer setup <source> <target> [options]

  <source>        : (required) The directory that contains all the files downloaded 
                    with the download command.
  <target>        : (required)The directory where Bigemap Server will be installed
                    on the host.

Options:

  --config <file>       : (optional) Pass in a JSON configuration file to configure the 
                          Bigemap Server password, port, and hostname.
  --self-signed, -l     : (optional) This will tell the installer to create self-signed 
                          SSL certificates and configure server to run over HTTPS.
                          Cannot be used with --ssl or --http.
  --ssl, -s <path>      : (optional) The path to a directory that contains SSL certificate files
                          1. cert.pem (a certificate file)
                          2. key.pem  (a private key for the certificate)
                          Cannot be used with --http or --self-signed.
  --ssl-key             : (optional) The private key file for the certificate
                          Must be used with --ssl-cert.
  --ssl-cert            : (optional) The certificate file
                          Must be used with --ssl-key.
  --ssl-ca              : (optional) The certificate file.
                          Cannot be used with --http, --ssl or --self-signed.

Example:

  ./bm-server-installer setup bm-server-files/ bm-server/ --config /usr/local/etc/bm-server.json

  ./bm-server-installer setup bm-server-files/ bm-server/ --ssl path/to/certs

Configuration:

  {
    \"Hostname\": \"localhost\",
    \"Port\": 3000,
    \"ServicePort\": 3001,
    \"Password\": \"password\",
    \"TilesetPath\": \"./tilesets\"
  }
"
}

help_settings() {
  echo -e "
If you want to change the config file or the server protocol (HTTP/HTTPS) after you set up Bigemap Server, use the settings command.

Usage:

  ./bm-server-installer settings <target> --config <path to JSON file>

  <target>        : (required)The directory where Bigemap Server will be installed
                    on the host.

Options:

  --config <file>       : (optional) Pass in a JSON configuration file to configure the 
                          Bigemap Server password, port, and hostname.
  --http                : (optional) Configure server to use HTTP. 
                          Cannot be used with --ssl or --self-signed
  --self-signed, -l     : (optional) This will tell the installer to create self-signed 
                          SSL certificates and configure server to run over HTTPS.
                          Cannot be used with --ssl or --http.
  --ssl, -s <path>      : (optional) The path to a directory that contains SSL certificate files
                          1. cert.pem (a certificate file)
                          2. key.pem  (a private key for the certificate)
                          Cannot be used with --http or --self-signed.
  --ssl-key             : (optional) The private key file for the certificate
                          Must be used with --ssl-cert.
  --ssl-cert            : (optional) The certificate file
                          Must be used with --ssl-key.
  --ssl-ca              : (optional) The certificate file.
                          Cannot be used with --http, --ssl or --self-signed.
  --network             : (optional) Set network mode, 'bridge' or 'host'.
  --init-auth           : (optional) Initialize auth app.
  --remove-auth         : (optional) Remove auth app.

Example:
  # Stop Bigemap Server
  (cd bm-server && ./bm-server.sh stop)

  # Update the server config
  ./bm-server-installer settings bm-server/ --config path/to/config.json

  # Switch to HTTP
  ./bm-server-installer settings bm-server/ --http

  # Switch to HTTPS (custom certificate)
  ./bm-server-installer settings bm-server/ --ssl path/to/certs
  
  # Switch to HTTPS (self-signed certificate)
  ./bm-server-installer settings bm-server/ --self-signed

  # Start Bigemap Server
  (cd bm-server && ./bm-server.sh start)

Configuration (JSON file):

  {
    \"Hostname\": \"localhost\",
    \"Port\": 3000,
    \"ServicePort\": 3001,
    \"Password\": \"password\",
    \"TilesetPath\": \"./tilesets\"
  }
"
}

DEFAULT_HOST="localhost"
DEFAULT_TILESET_PATH="./tilesets"
DEFAULT_PORT="3000"
DEFAULT_SPORT="3001"
DEFAULT_AUTH_PORT="4388"
DEFAULT_PWD="password"
DEFAULT_NETWORK_MODE="bridge"

HOST=$DEFAULT_HOST
PORT=$DEFAULT_PORT
SPORT=$DEFAULT_SPORT
AUTH_APP_PORT=$DEFAULT_AUTH_PORT
PWD=$DEFAULT_PWD
TILESET_PATH=$DEFAULT_TILESET_PATH
#USE_HTTP=1

getopts() {

  while [ $# != 0 ]; do
    case $1 in
    "--config" | "-c")
      CONF_FILE="$2"
      if [ ! -f $CONF_FILE ]; then
        echo -e "${RED}Error arg for $1: '$CONF_FILE' is incorrect.${NC}"
        exit 1
      fi
      shift 2
      ;;
    "--ssl" | "-s")
      USE_HTTP=0
      CERTS_PATH="$2"
      if [ ! -d $CERTS_PATH ]; then
        echo -e "${RED}Error arg for $1: '$CERTS_PATH' is incorrect.${NC}"
        exit 1
      fi
      shift 2
      ;;
    "--ssl-key")
      USE_HTTP=0
      CERTS_KEY="$2"
      if [ ! -f $CERTS_KEY ]; then
        echo -e "${RED}Error arg for $1: '$CERTS_KEY' is incorrect.${NC}"
        exit 1
      fi
      shift 2
      ;;
    "--ssl-cert")
      USE_HTTP=0
      CERTS_CERT="$2"
      if [ ! -f $CERTS_CERT ]; then
        echo -e "${RED}Error arg for $1: '$CERTS_CERT' is incorrect.${NC}"
        exit 1
      fi
      shift 2
      ;;
    "--ssl-ca")
      USE_HTTP=0
      CERTS_CA="$2"
      if [ ! -f $CERTS_CA ]; then
        echo -e "${RED}Error arg for $1: '$CERTS_CA' is incorrect.${NC}"
        exit 1
      fi
      shift 2
      ;;
    "--self-signed" | "-l")
      USE_HTTP=0
      SELF_SIGNED=1
      shift
      ;;
    "--http")
      USE_HTTP=1
      shift
      ;;
    "--init-auth")
      INIT_BM_AUTH=1
      shift
      ;;
    "--remove-auth")
      RM_BM_AUTH=1
      shift
      ;;
    "--network")
      SET_NETWORK_MODE="$2"
      if [[ "$SET_NETWORK_MODE" != "bridge" ]] && [[ "$SET_NETWORK_MODE" != "host" ]]; then
        echo -e "${RED} Unknown network mode '$SET_NETWORK_MODE'.${NC}"
        exit 1
      fi
      shift 2
      ;;
    *)
      echo "Error option $1"
      case $SOURCE in
      'settings') help_settings ;;
      *) help_setup ;;
      esac
      exit 1
      ;;
    esac
  done

  if [[ -n "$CERTS_KEY" && ! -n "$CERTS_CERT" ]] || [[ ! -n "$CERTS_KEY" && -n "$CERTS_CERT" ]]; then
    echo -e "${RED}Both of options --ssl-key, --ssl-cert must be used.${NC}"
    exit 1
  fi

  if [ -f "$CONF_FILE" ]; then
    HOST=$(cat $CONF_FILE | grep -i 'Hostname' | grep -Eo ':[" ]?*[^"]+' | grep -Eo '[^: "]+' || echo $DEFAULT_HOST)
    PORT=$(cat $CONF_FILE | grep -i '"Port"' | grep -Eo ':[" ]?*[^"]+' | grep -Eo '[^: ",]+' || echo $DEFAULT_PORT)
    SPORT=$(cat $CONF_FILE | grep -i 'ServicePort' | grep -Eo ':[" ]?*[^"]+' | grep -Eo '[^: ",]+' || echo $DEFAULT_SPORT)
    AUTH_APP_PORT=$(cat $CONF_FILE | grep -i 'AuthPort' | grep -Eo ':[" ]?*[^"]+' | grep -Eo '[^: ",]+' || echo $DEFAULT_AUTH_PORT)
    PWD=$(cat $CONF_FILE | grep -i 'Password' | grep -Eo ':[" ]?*[^"]+' | grep -Eo '[^: "]+' || echo $DEFAULT_PWD)
    TILESET_PATH=$(cat $CONF_FILE | grep -i 'TilesetPath' | grep -Eo ':[" ]?*[^"]+' | grep -Eo '[^: "]+' || echo $DEFAULT_TILESET_PATH)
  fi
}

apply_https() {
  mkdir -p "$TARGET/certs" || exit 1
  if [ "$SELF_SIGNED" = "1" ]; then
    echo '--------------  Create self-signed SSL certificates --------------'
    self_signed $TARGET/certs
    if [ $? -ne 0 ]; then
      echo "----------------- Create self-signed SSL failed! -----------------"
      exit 1
    fi
    echo "---------------- Create self-signed SSL compiled! ----------------"
  elif [ -n $CERTS_PATH ]; then
    cp -f $CERTS_PATH/key.pem $TARGET/certs/key.pem
    cp -f $CERTS_PATH/cert.pem $TARGET/certs/cert.pem
    echo "Copy SSL certificate files compiled!"
  elif [ -n $CERTS_KEY && -n $CERTS_CERT ]; then
    cp -f $CERTS_KEY $TARGET/certs/key.pem
    cp -f $CERTS_CERT $TARGET/certs/cert.pem
    if [ -n $CERTS_CA ]; then
      cp -f $CERTS_CA $TARGET/certs/ca-cert.pem
    fi
  fi
}

settings() {
  if [ ! -n "$1" ]; then
    echo -e "${RED}You must specify the directory where Bigemap Server is installed. ${NC}\n"
    help_settings
    exit 1
  fi

  TARGET=$1; shift

  if [ $# = 0 ]; then
    echo -e "${RED}You does not specify [OPTIONS]. ${NC}\n"
    help_settings
    exit 1
  fi

  if [ ! -f "$TARGET/docker-compose.yml" ]; then
    echo -e "${RED} '$TARGET' does not contain Bigemap Server. ${NC}\n"
    exit 1
  fi

  HOST=$(parse_yaml $TARGET/docker-compose.yml | grep -Eo 'BS_HOST=*[^"]+' | cut -d'=' -f2)
  PORT=$(parse_yaml $TARGET/docker-compose.yml | grep -Eo 'BS_PORT=*[^"]+' | cut -d'=' -f2)
  SPORT=$(parse_yaml $TARGET/docker-compose.yml | grep -Eo 'BS_SPORT=*[^"]+' | cut -d'=' -f2)
  AUTH_APP_PORT=$(echo $(parse_yaml $TARGET/docker-compose.yml | grep -Eo 'BA_PORT=*[^"]+' || echo $DEFAULT_AUTH_PORT) | cut -d'=' -f2)
  PWD=$(parse_yaml $TARGET/docker-compose.yml | grep -Eo 'BS_PWD=*[^"]+' | cut -d'=' -f2)
  TILESET_PATH=$(echo $(parse_yaml $TARGET/docker-compose.yml | grep -Eo "[^'\"]+:/usr/local/tilesets" || echo $DEFAULT_TILESET_PATH) | cut -d":" -f1)
  
  USE_HTTP_SETTED=$(echo $(parse_yaml $TARGET/docker-compose.yml | grep -Eo 'BS_USEHTTP=*[^"]+' || echo '1') | cut -d'=' -f2)
  NETWORK_MODE_SETTED=$(echo $(parse_yaml $TARGET/docker-compose.yml | grep network_mode || echo 'bridge') | cut -d'"' -f2)

  getopts $*

  # network_mode
  if [ -n "$SET_NETWORK_MODE" ] && [ "$NETWORK_MODE_SETTED" != "$SET_NETWORK_MODE" ]; then
    SLN=$(cat -n $TARGET/docker-compose.yml | grep ports: | awk '{print $1}')
    ELN=$[$(cat -n $TARGET/docker-compose.yml | grep environment: | awk '{print $1}')-1]

    if [ "$SET_NETWORK_MODE" = "host" ]; then
      sed -i "${SLN},${ELN}s/^/#/g" $TARGET/docker-compose.yml

      NETWORK_MODE_CFG="   network_mode: host"
      LN=$(cat -n $TARGET/docker-compose.yml | grep network_mode | awk '{print $1}')
      if [ -z "$LN" ]; then
        LN=$[$(cat -n $TARGET/docker-compose.yml | grep volumes: | awk '{print $1}')-1]
        sed -i "${LN}i\ ${NETWORK_MODE_CFG}" $TARGET/docker-compose.yml;
      else
        sed -i "${LN}s/^#//g" $TARGET/docker-compose.yml
      fi

      if [ "$PORT" != "$DEFAULT_PORT" ]; then
        LN=$[$(cat -n $TARGET/docker-compose.yml | grep environment: | awk '{print $1}')+1]
        sed -i "${LN}i\      - DOCKER_PORT=$PORT" $TARGET/docker-compose.yml;
      fi
      if [ "$SPORT" != "$DEFAULT_SPORT" ]; then
        LN=$[$(cat -n $TARGET/docker-compose.yml | grep environment: | awk '{print $1}')+1]
        sed -i "${LN}i\      - DOCKER_SPORT=$SPORT" $TARGET/docker-compose.yml;
      fi
    else
      sed -i "${SLN},${ELN}s/^#//g" $TARGET/docker-compose.yml
      sed_d $TARGET/docker-compose.yml DOCKER_PORT DOCKER_SPORT network_mode
    fi

    NETWORK_MODE_SETTED=$SET_NETWORK_MODE
  fi

  # vols
  sed -i "s@'.*:/usr/local/tilesets'@'$TILESET_PATH:/usr/local/tilesets'@" $TARGET/docker-compose.yml
  
  # ports
  sed -i "s/[0-9]*:3000/$PORT:3000/" $TARGET/docker-compose.yml
  sed -i "s/[0-9]*:3001/$SPORT:3001/" $TARGET/docker-compose.yml
  sed -i "s/:[0-9]\+/:$PORT/" $TARGET/bm-server.sh

  # environments
  sed -i "s/BS_HOST=.*/BS_HOST=$HOST/" $TARGET/docker-compose.yml
  sed -i "s/BS_PWD=.*/BS_PWD=$PWD/" $TARGET/docker-compose.yml
  sed -i "s/BS_PORT=.*/BS_PORT=$PORT/" $TARGET/docker-compose.yml
  sed -i "s/BS_SPORT=.*/BS_SPORT=$SPORT/" $TARGET/docker-compose.yml
  sed -i "s/BA_PORT=.*/BA_PORT=$AUTH_APP_PORT/" $TARGET/docker-compose.yml

  # https
  if [ -n "$USE_HTTP" ] && [ "$USE_HTTP_SETTED" != "$USE_HTTP" ]; then
    if [ "$USE_HTTP" = "0" ]; then
      apply_https

      local VOL_CERTS="     - './certs:/usr/local/certs'"
      LN=$(cat -n $TARGET/docker-compose.yml | grep '/usr/local/certs' | awk '{print $1}')
      if [ -z "$LN" ]; then 
        LN=$[$(cat -n $TARGET/docker-compose.yml | grep volumes: | awk '{print $1}')+1]
        sed -i "${LN}i\ ${VOL_CERTS}" $TARGET/docker-compose.yml;
      fi
    else
      sed_d $TARGET/docker-compose.yml '/usr/local/certs'
    fi

    sed -i "s/BS_USEHTTP=.*/BS_USEHTTP=$USE_HTTP/" $TARGET/docker-compose.yml
  fi

  # remove & init auth
  if [[ "$INIT_BM_AUTH" = "1" ]] || [[ "$RM_BM_AUTH" = "1" ]]; then
    sed_d $TARGET/docker-compose.yml bm-auth BA_PORT 4388 $AUTH_APP_PORT

    if [ "$INIT_BM_AUTH" = "1" ]; then
      AUTH_APP_VERSION=$(find $TARGET/docker -name "bm-auth-*.tar.gz" | grep -Eo "v([0-9]+\.)+[0-9]+" | sort -V | tail -n 1)
      if [ "$AUTH_APP_VERSION" != "" ]; then
        echo -e "Extract bm-auth files...\n"
        AUTH_APP="bm-auth-$AUTH_APP_VERSION.tar.gz"
        if [ -f "$TARGET/docker/$AUTH_APP" ]; then
          mkdir -p "$TARGET/bm-auth" || exit 1
          tar zxf $TARGET/docker/$AUTH_APP -C $TARGET/bm-auth || exit 1

          local VOL_AUTH_APP="     - './bm-auth:/usr/local/bm-auth'"
          local PORTS_AUTH_APP="     - '$AUTH_APP_PORT:$AUTH_APP_PORT'"
          local BA_PORT_ENV="     - BA_PORT=$AUTH_APP_PORT"

          # vol
          LN=$[$(cat -n $TARGET/docker-compose.yml | grep volumes: | awk '{print $1}')+1]
          sed -i "${LN}i\ ${VOL_AUTH_APP}" $TARGET/docker-compose.yml;

          # port
          LN=$[$(cat -n $TARGET/docker-compose.yml | grep ports: | awk '{print $1}')+1]
          sed -i "${LN}i\ ${PORTS_AUTH_APP}" $TARGET/docker-compose.yml;
          if [ "$NETWORK_MODE_SETTED" = "host" ]; then
            sed -i "${LN}s/^/#/g" $TARGET/docker-compose.yml
          fi

          # env
          LN=$[$(cat -n $TARGET/docker-compose.yml | grep environment: | awk '{print $1}')+1]
          sed -i "${LN}i\ ${BA_PORT_ENV}" $TARGET/docker-compose.yml;
        fi
      fi
    fi
  fi

  echo -e "${GREEN}Bigemap Server settings completed.${NC}"
  echo -e "Now you can run (cd bm-server && ./bm-server.sh restart) to restart server."
}

setup() {
  if [ ! -n "$1" ]; then
    echo -e "${RED}You must specify the directory that contains all the files downloaded. ${NC}\n"
    help_setup
    exit 1
  fi

  if [ ! -n "$2" ]; then
    echo -e "${RED}You must specify the directory where Bigemap Server will be installed. ${NC}\n"
    help_setup
    exit 1
  fi

  SOURCE=$1
  TARGET=$2
  shift 2

  if [ ! -n "$(find $SOURCE/docker -name "bm-server-*.tar.gz" | grep -Eo "v([0-9]+\.)+[0-9]+-[a-z0-9]+" | sort -V | tail -n 1)" ]; then
    echo -e "${RED}You must run download command first.\nor the source directory is incorrect.${NC}"
    exit 1
  fi

  getopts $*

  VOL_CERTS=""
  [ -z $USE_HTTP ] && USE_HTTP=1
  if [ "$USE_HTTP" = "0" ]; then
    apply_https
    VOL_CERTS="
      - './certs:/usr/local/certs'"
  fi

  mkdir -p "$TARGET/.bm-server" || exit 1
  mkdir -p "$TARGET/docker" || exit 1
  if [ "$TILESET_PATH" = "$DEFAULT_TILESET_PATH" ]; then
    mkdir -p "$TARGET/tilesets" || exit 1
  else
    mkdir -p "$TILESET_PATH" || exit 1
  fi

  echo -e "Loading docker images...\n"

  TARGET_VERSION=$(find $SOURCE/docker -name "bm-server-*.tar.gz" | grep -Eo "v([0-9]+\.)+[0-9]+-[a-z0-9]+" | sort -V | tail -n 1)
  DOCKER_IMAGE="bm-server-$TARGET_VERSION-$BM_ARCH.tar.gz"

  docker load -i $SOURCE/docker/$DOCKER_IMAGE
  if [ $? -ne 0 ]; then
    echo -e "${RED}Docker load image failed!${NC}"
    exit 1
  fi

  mv -f $SOURCE/docker/* $TARGET/docker/
  echo -e "\n$DOCKER_IMAGE load completed."

  echo "
version: '2'
services:
  bm-server:
    image: bigemap/bm-server:$TARGET_VERSION
    container_name: bm-server
    restart: always
    privileged: true
    volumes:
      - './.bm-server:/usr/local/bm-server'
      - '$TILESET_PATH:/usr/local/tilesets'$VOL_CERTS
    ports:
      - '$PORT:3000'
      - '$SPORT:3001'
    environment:
      - BS_HOST=$HOST
      - BS_PWD=$PWD
      - BS_PORT=$PORT
      - BS_SPORT=$SPORT
      - BS_USEHTTP=$USE_HTTP
    devices:
      - '/dev/usb:/dev/usb'
      - '/dev/bus/usb:/dev/bus/usb'
" >$TARGET/docker-compose.yml

  VAR1='$1'
  VAR2='$?'

  case $USE_HTTP in
  1) PROTOCOL='http' ;;
  0) PROTOCOL='https' ;;
  esac

  echo "
#!/bin/bash
#
# Welcom to use Bigemap Server
# Copyright(c) 2019 BIGEMAP
#

start() {
  docker-compose up -d

  if [ $VAR2 -ne 0 ]; then
    exit 1
  fi

  echo '
Bigemap Server running ...

  $PROTOCOL://<your-ip-address>:$PORT
  
Default:

  $PROTOCOL://localhost:$PORT
'
}

stop() {
  docker-compose stop
}

logs() {
  docker-compose logs
}

version() {
  echo \"$TARGET_VERSION\"
}

usage() {
  echo \"
Welcom to use Bigemap Server
For more infomation, please visit http://www.bigemap.com/offlinemaps
Version: $TARGET_VERSION

Usage: ./bm-server [command]

Commands:
  start               Start Bigemap Server
  stop                Stop Bigemap Server
  restart             Restart Bigemap Server
  logs                View Bigemap Server logs

Notes:

  Publish offline map services:
  Copy the directory downloaded by Bigemap into $TARGET/tilesets
  see also http://www.bigemap.com/helps/doc20190104122.html

\"
}

case $VAR1 in
\"start\") start ;;
\"stop\") stop ;;
\"restart\") 
  stop
  start
  ;;
\"logs\") logs ;;
\"version\") version ;;
*) usage ;;
esac
" >$TARGET/bm-server.sh

  chmod +x $TARGET/bm-server.sh
  chmod +x $TARGET/docker-compose.yml

  rm -rf $SOURCE
  echo -e "${GREEN}Bigemap Server setup completed.${NC}"
  echo -e "Now you can run (cd bm-server && ./bm-server.sh start) to start server."
}
